@@ -23,6 +23,16 @@ class Redis_Cluster_Test extends Redis_Test {
         RedisCluster::FAILOVER_DISTRIBUTE
     );
 
+    /**
+     * @var string
+     */
+    protected $sessionPrefix = 'PHPREDIS_CLUSTER_SESSION:';
+
+    /**
+     * @var string
+     */
+    protected $sessionSaveHandler = 'rediscluster';
+
     /* Tests we'll skip all together in the context of RedisCluster.  The 
      * RedisCluster class doesn't implement specialized (non-redis) commands
      * such as sortAsc, or sortDesc and other commands such as SELECT are
@@ -34,6 +44,22 @@ class Redis_Cluster_Test extends Redis_Test {
     public function testReconnectSelect() { return $this->markTestSkipped(); }
     public function testMultipleConnect() { return $this->markTestSkipped(); }
     public function testDoublePipeNoOp() { return $this->markTestSkipped(); }
+    public function testSwapDB() { return $this->markTestSkipped(); }
+    public function testConnectException() { return $this->markTestSkipped(); }
+
+    /* Session locking feature is currently not supported in in context of Redis Cluster.
+       The biggest issue for this is the distribution nature of Redis cluster */
+    public function testSession_lockKeyCorrect() { return $this->markTestSkipped(); }
+    public function testSession_lockingDisabledByDefault() { return $this->markTestSkipped(); }
+    public function testSession_lockReleasedOnClose() { return $this->markTestSkipped(); }
+    public function testSession_ttlMaxExecutionTime() { return $this->markTestSkipped(); }
+    public function testSession_ttlLockExpire() { return $this->markTestSkipped(); }
+    public function testSession_lockHoldCheckBeforeWrite_otherProcessHasLock() { return $this->markTestSkipped(); }
+    public function testSession_lockHoldCheckBeforeWrite_nobodyHasLock() { return $this->markTestSkipped(); }
+    public function testSession_correctLockRetryCount() { return $this->markTestSkipped(); }
+    public function testSession_defaultLockRetryCount() { return $this->markTestSkipped(); }
+    public function testSession_noUnlockOfOtherProcess() { return $this->markTestSkipped(); }
+    public function testSession_lockWaitTime() { return $this->markTestSkipped(); }
 
     /* Load our seeds on construction */
     public function __construct() {
@@ -574,8 +600,8 @@ class Redis_Cluster_Test extends Redis_Test {
 
     public function testSession()
     {
-        ini_set('session.save_handler', 'rediscluster');
-        ini_set('session.save_path', implode('&', array_map(function ($seed) {
+        @ini_set('session.save_handler', 'rediscluster');
+        @ini_set('session.save_path', implode('&', array_map(function ($seed) {
             return 'seed[]=' . $seed;
         }, self::$_arr_node_map)) . '&failover=error');
         if (!@session_start()) {
@@ -584,5 +610,17 @@ class Redis_Cluster_Test extends Redis_Test {
         session_write_close();
         $this->assertTrue($this->redis->exists('PHPREDIS_CLUSTER_SESSION:' . session_id()));
     }
+
+    /**
+     * @inheritdoc
+     */
+    protected function getFullHostPath()
+    {
+        $hosts = array_map(function ($host) {
+            return 'seed[]=' . $host . '';
+        }, self::$_arr_node_map);
+
+        return implode('&', $hosts);
+    }
 }
 ?>
